﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;

namespace ChemicalC
{
    public partial class Form1 : Form
    {
        public Recipes recipes = new Recipes();
        public Form1()
        {
            InitializeComponent();
            if (!recipes.readFromXML())
                recipes.writeToXML();//если не удалось прочитать файл, считаем, что его нет и создаем новый
            bindingSourceRecipe.DataSource = recipes.recipe;
            try
            {
                bindingSourceRecipe.Position = recipes.last_recipe;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error");
            }
            dataGridView1.Enabled = cb_recipes.Enabled = bindingSourceRecipe.Count > 0;
        }
        private void rb_form_CheckedChanged(object sender, EventArgs e)
        {
            label2.Visible = size2.Visible = lb_un2.Visible = rb_form.Checked;
            label1.Text = rb_form.Checked ? "Ширина" : "Диаметр";
            Calc(sender, e);
        }
        private void Calc(object sender, EventArgs e)
        {
            int maxwidth = 0;
            float total = 0;
            List<int> w = new List<int>();
            List<float> v = new List<float>();
            bool error = false;
            List<Recipes.Chemical> l = new List<Recipes.Chemical>();

            Recipes.Recipe r = (Recipes.Recipe)bindingSourceRecipe.Current;

            if (r == null) return;
            if (r.chemicals.Count == 0) return;

            float sq = 0;
            float k = 0;

            try
            {
                sq = float.Parse(size1.Text);
                if (rb_form.Checked)
                    sq *= float.Parse(size2.Text);
                else
                    sq *= sq / 4 * (float)Math.PI;
            }
            catch { error = true; }

            try
            {
                k = float.Parse(r.base_area) / sq;
            }
            catch { error = true; }

            for (int i = 0; i < r.chemicals.Count; i++)
            {
                if (r.chemicals[i].chemical_name == null || r.chemicals[i].chemical_name?.Trim() == "") continue;
                if (float.TryParse(r.chemicals[i].chemical_value, out float a))
                {
                    float aa = k != 0? a / k : 0;
                    v.Add(aa);
                    total += aa;
                }
                else
                {
                    v.Add(0);
                    error = true;
                }
                int ww = TextRenderer.MeasureText(r.chemicals[i].chemical_name, tb_result.Font).Width;
                w.Add(ww);
                maxwidth = Math.Max(maxwidth, ww);
            }

            tb_result.Text = error ? "Внимание! При обработке возникли ошибки." + Environment.NewLine + "Проверьте корректность исходных данных: " + Environment.NewLine + "--------------------------" + Environment.NewLine : "";
            tb_result.Text += "Площадь: " + sq.ToString() + " " + r.size_units + "²" + Environment.NewLine +
                "Кол-во химикатов всего: " + total + " " + r.value_units + Environment.NewLine + "-----------------------------------";

            maxwidth += TextRenderer.MeasureText("...", tb_result.Font).Width;

            for (int i = 0; i < r.chemicals.Count; i++)
            {
                if (r.chemicals[i].chemical_name == null || r.chemicals[i].chemical_name?.Trim() == "") continue;
                tb_result.Text += Environment.NewLine;
                tb_result.Text += r.chemicals[i].chemical_name;
                string tab = "...";
                while (TextRenderer.MeasureText(tab, tb_result.Font).Width + w[i] < maxwidth)
                    tab += ".";
                tb_result.Text += "  "+tab + "  "+v[i].ToString() + ", " + r.value_units;
            }
        }
        private void toolStrip1_ItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {
            switch (e.ClickedItem.Name)
            {
                case "tsb_add":
                    Recipes.Recipe r1 = new Recipes.Recipe();
                    Form2 f_add = new Form2(r1) { Text = "Добавление" };
                    if (f_add.ShowDialog() == DialogResult.OK)
                    {
                        bindingSourceRecipe.Add(r1);
                        bindingSourceRecipe.MoveLast();
                    }
                    break;
                case "tsb_edit":
                    Form2 f_edit = new Form2((Recipes.Recipe)bindingSourceRecipe.Current) { Text = "Изменение" };
                    if (f_edit.ShowDialog() == DialogResult.OK)
                        bindingSourceRecipe.EndEdit();
                    else
                        bindingSourceRecipe.CancelEdit();
                    break;
                case "tsb_del":
                    if (bindingSourceRecipe.Count == 0) return;
                    Form3 form3 = new Form3(cb_recipes.Text);
                    if (form3.ShowDialog() == DialogResult.OK) bindingSourceRecipe.RemoveCurrent();
                    break;
                case "about":
                    about ab = new about();
                    ab.ShowDialog();
                    break;
            }
            dataGridView1.Enabled = cb_recipes.Enabled = bindingSourceRecipe.Count > 0;
            recipes.writeToXML();
        }
        private void cb_recipes_DrawItem(object sender, DrawItemEventArgs e)
        {
            SolidBrush b = new SolidBrush(Color.FromArgb(192, 255, 192));
            e.Graphics.FillRectangle((e.State & DrawItemState.Focus) == DrawItemState.None ? Brushes.White : b, e.Bounds);
            if (e.Index > -1 && cb_recipes.Items.Count > 0)
                e.Graphics.DrawString(cb_recipes.GetItemText(cb_recipes.Items[e.Index]), e.Font, Brushes.Black, e.Bounds.X, e.Bounds.Y);
            b.Dispose();
        }
        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            recipes.last_recipe = bindingSourceRecipe.Position;
            recipes.writeToXML();
        }
        private bool hasDot = false;
        private void floatonly_KeyPress(object sender, KeyPressEventArgs e)
        {
            e.Handled = !char.IsControl(e.KeyChar) && !char.IsDigit(e.KeyChar) && !(e.KeyChar == '.' && hasDot);
        }
        private void floatonly_TextChanged(object sender, EventArgs e)
        {
            hasDot = ((TextBox)sender).Text.IndexOf('.') < 0;
            if ((string)((TextBox)sender).Tag == "calc") Calc(sender, e);
        }
        private void dataGridView1_EditingControlShowing(object sender, DataGridViewEditingControlShowingEventArgs e)
        {
            if (dataGridView1.CurrentCell.ColumnIndex == 1)
            {
                e.Control.KeyPress += new KeyPressEventHandler(floatonly_KeyPress);
                e.Control.TextChanged += new EventHandler(floatonly_TextChanged);
            }
            else
            {
                e.Control.KeyPress -= (KeyPressEventHandler)(floatonly_KeyPress);
                e.Control.TextChanged -= (EventHandler)(floatonly_TextChanged);
            }
        }
        private void dataGridView1_DataBindingComplete(object sender, DataGridViewBindingCompleteEventArgs e)
        {
            Calc(sender, e);
        }
    }
}
